/**
 * @author acvetkov@yandex-team.ru
 * @overview Wrap helper
 */

'use strict';

var Q = require('q');

/**
 * Wrap all methods from namespace
 * @param {String} namespace
 * @param {Array} asyncMethods
 * @param {Array} [syncMethods]
 * @returns {Object}
 */
module.exports = function (namespace, asyncMethods) {
    var syncMethods = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

    var asyncObject = asyncMethods.reduce(function (result, method) {
        if (exists(namespace, method)) {
            result[method] = wrapAsync(namespace, method);
        }
        return result;
    }, {});

    var syncObject = syncMethods.reduce(function (result, method) {
        if (exists(namespace, method)) {
            result[method] = wrapSync(namespace, method);
        }
        return result;
    }, {});

    return merge(asyncObject, syncObject);
};

/**
 * Check method exists
 * @param {String} namespace
 * @param {String} method
 * @returns {Boolean}
 */
function exists(namespace, method) {
    return Boolean(chrome[namespace]) && Boolean(chrome[namespace][method]);
}

/**
 * Wrap async method
 * @param {String} namespace
 * @param {String} method
 * @returns {Function}
 */
function wrapAsync(namespace, method) {
    return function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        var defer = Q.defer();
        args.push(getResolver(defer));
        Function.prototype.apply.call(chrome[namespace][method], chrome[namespace], args);
        return defer.promise;
    };
}

/**
 * @param {deferred} defer
 * @returns {Function}
 */
function getResolver(defer) {
    return function () {
        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
        }

        if (chrome.runtime.lastError) {
            return defer.reject(chrome.runtime.lastError);
        }
        return Function.prototype.apply.call(defer.resolve, defer, args);
    };
}

/**
 * Wrap sync method
 * @param {String} namespace
 * @param {String} method
 * @returns {Function}
 */
function wrapSync(namespace, method) {
    return function () {
        for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
            args[_key3] = arguments[_key3];
        }

        try {
            return Q.resolve(Function.prototype.apply.call(chrome[namespace][method], chrome[namespace], args));
        } catch (e) {
            return Q.reject(e);
        }
    };
}

/**
 * Merge objects
 * @param {Object} asyncObj
 * @param {Object} syncObj
 * @returns {Object}
 */
function merge(asyncObj, syncObj) {
    return Object.keys(syncObj).reduce(function (result, key) {
        result[key] = syncObj[key];
        return result;
    }, asyncObj);
}